@@ -9,7 +9,7 @@
 #
 
 
-__version__ = "0.12.0"
+__version__ = "0.14.0"
 
 
 __license__ = "AGPLv3"
@@ -20,7 +20,30 @@ aiootp - an asynchronous one-time-pad based crypto and anonymity library.
 """
 
 
-DEBUG_MODE = False
+class DebugControl:
+    """
+    Enabling debugging reveals omitted values in object ``repr``s &
+    turns on asyncio's debugging.
+    """
+
+    _switches = []
+    _DEBUG_MODE = False
+
+    @classmethod
+    def is_debugging(cls):
+        return cls._DEBUG_MODE
+
+    @classmethod
+    def enable_debugging(cls):
+        cls._DEBUG_MODE = True
+        for toggle in cls._switches:
+            toggle()
+
+    @classmethod
+    def disable_debugging(cls):
+        cls._DEBUG_MODE = False
+        for toggle in cls._switches:
+            toggle()
 
 
 from .paths import *
@@ -46,5 +69,7 @@ __all__ = [
 ]
 
 
+del __datasets
+del __aiocontext
 del __ui_coordination
 
