@@ -13,12 +13,12 @@ __all__ = [
     "commons",
     "bits",
     "primes",
-    "power10",
     "Namespace",
     "WORD_LIST",
     "ASCII_TABLE",
     "BYTES_TABLE",
     "BASE_64_TABLE",
+    "ASCII_TABLE_128",
     "ONION_CHAR_TABLE",
     "ASCII_ALPHANUMERIC",
 ]
@@ -34,9 +34,10 @@ import sys
 import copy
 import types
 import asyncio
+from hashlib import sha3_256
 from os import linesep
 from .__datasets import *
-from . import DEBUG_MODE
+from . import DebugControl
 
 
 async def aimport_namespace(
@@ -114,7 +115,7 @@ async def aredact_keys(string, *, rule=r"[0-9a-fA-F]{64,}"):
     key_finder = re.compile(rule)
     for key in key_finder.finditer(string):
         await asyncio.sleep(0)
-        string = string.replace(key.group(0), "<omitted-key>")
+        string = string.replace(key.group(0), OMITTED)
     return string
 
 
@@ -125,7 +126,7 @@ def redact_keys(string, *, rule=r"[0-9a-fA-F]{64,}"):
     """
     key_finder = re.compile(rule)
     for key in key_finder.finditer(string):
-        string = string.replace(key.group(0), "<omitted-key>")
+        string = string.replace(key.group(0), OMITTED)
     return string
 
 
@@ -173,15 +174,22 @@ class Namespace:
         """
         return len(self.__dict__)
 
-    def __str__(self, *, debugging=DEBUG_MODE, tab=4 * " "):
+    def __str__(self, *, tab=4 * " "):
         """
         Pretty displays the Namespace's mapping.
         """
         result = self.__class__.__qualname__
         result += f"({linesep}" + "    mapping={"
-        for variable, value in self:
-            result += f"{linesep + 2 * tab}{variable}:\t{repr(value)},"
-        return redact_keys(result + f"{linesep}" + "    }" + f"{linesep})")
+        ending = f"{linesep}" + "    }" + f"{linesep})"
+        spacer = f"{linesep + 2 * tab}"
+        if DebugControl.is_debugging():
+            for variable, value in self:
+                result += spacer + f"{variable}:\t{repr(value)},"
+        else:
+            for variable, value in self:
+                omitted_value = f"{OMITTED} of {type(value)}"
+                result += spacer + f"{variable}:\t{omitted_value},"
+        return result + ending
 
     def __repr__(self):
         """
@@ -257,6 +265,7 @@ __extras = {
     "AGE": "age_of_connection",
     "ASCII_ALPHANUMERIC": ASCII_ALPHANUMERIC,
     "ASCII_TABLE": ASCII_TABLE,
+    "ASCII_TABLE_128": ASCII_TABLE_128,
     "BASE_64_TABLE": BASE_64_TABLE,
     "BYTES_TABLE": BYTES_TABLE,
     "CHANNEL": "channel",
@@ -281,11 +290,15 @@ __extras = {
     "KEY": "key",
     "KEY_ID": "key_id",
     "KEYED_PASSWORD": "keyed_password",
+    "LIST_ENCODING": "listed_ciphertext",
     "LISTENING": "listening",
     "MAINTAINING": "maintaining",
+    "MANIFEST": sha3_256(f"__manifest__{NONE}".encode()).hexdigest(),
+    "MAP_ENCODING": "mapped_ciphertext",
     "MESSAGE_ID": "message_id",
     "MESSAGES": "message_archive",
     "METADATA": "metadata",
+    "METATAG": sha3_256(f"__metatags__{NONE}".encode()).hexdigest(),
     "Namespace": Namespace,
     "NEW_CONTACT": "new_contact",
     "NEXT_KEYED_PASSWORD": "next_keyed_password",
@@ -294,6 +307,7 @@ __extras = {
     "NUM": NUM,
     "OLD_KEY": "last_shared_key",
     "OLD_VERIFID": "last_verification_code",
+    "OMITTED": "<omitted-data>",
     "ONION": "onion",
     "ONION_CHAR_TABLE": ONION_CHAR_TABLE,
     "PASSWORD": "password",
@@ -344,7 +358,6 @@ __extras = {
     "aredact_keys": aredact_keys,
     "redact_keys": redact_keys,
     "bits": bits,
-    "power10": power10,
     "primes": primes,
 }
 
