@@ -9,7 +9,15 @@
 #
 
 
-__all__ = ["keygens", "AsyncKeys", "Keys", "amnemonic", "mnemonic"]
+__all__ = [
+    "keygens",
+    "AsyncKeys",
+    "Keys",
+    "amnemonic",
+    "mnemonic",
+    "akeypair",
+    "keypair",
+]
 
 
 __doc__ = """
@@ -21,14 +29,13 @@ from .asynchs import *
 from .commons import *
 from .randoms import salt
 from .randoms import asalt
+from .randoms import csprbg
 from .randoms import csprng
 from .randoms import acsprng
 from .randoms import token_bytes
 from .randoms import atoken_bytes
 from .ciphers import keys
 from .ciphers import akeys
-from .ciphers import subkeys
-from .ciphers import asubkeys
 from .ciphers import Passcrypt
 from .ciphers import passcrypt
 from .ciphers import apasscrypt
@@ -59,39 +66,30 @@ async def atable_key_gen(key=None, table=ASCII_TABLE):
     The ASCII_TABLE that's provided as a default, is a comprehensive set
     of ascii characters that are legible, unique, and have single octet
     byte representations. This ensures keys can be converted into a
-    cryptographically secure byte stream of binary numbers from 32 to
+    cryptographically secure byte stream of numbers from 32 to
     126.
 
     This generator function provides either deterministic keys from a
-    user key, or generates a random sha_512 hash key and derives a
-    random key with the desired table elements from this hash. The
-    result is a random, normal distribution of characters from among the
-    items within the table. The table argument can also contain an
-    aribtrary set of objects. In this case, this generator function is
-    essentially analogous to one that generates a series of random
-    choices from the table, with the added functionality of allowing
-    deterministic outputs that coincide with the key and size arguments.
+    user key, or generates a random 512-bit hash and derives a random
+    key with the desired table elements from this hash. The result is a
+    random, normal distribution of characters from among the items
+    within the table.
 
     Usage Examples:
 
     # To produce a 60 byte key of characters from the default table
     key = "hotdiggitydog_thischowisyummy"
-    new_key = ""
-    async for char in atable_key_gen(key=key)[:60]:
-        new_key += char
+    async with atable_key_gen(key=key) as generator:
+        new_key = await generator()
+        assert new_key != await generator()
     print(new_key)
     >>> Hx`4^ej;u&/]qOF21Ea2~(6f"smp'DvMk[(wy'lME%CpCo|1ZWt> &tu=Mw_
     """
-    if key == None:
+    if not key:
         key = await acsprng()
-    if not is_iterable(table):
-        raise TypeError("table is not iterable")
-    elif isinstance(table, dict):
-        table = list(table.keys())
-    prime = primes[32][0]
-    table_size = len(table)
-    async for index in akeys(key, key).aresize(16).aint(16):
-        yield table[index % prime % table_size]
+    size = len(table[:])
+    async for key_portion in akeys(key, key).aint(16).ato_base(size, table):
+        yield key_portion
 
 
 @comprehension()
@@ -106,40 +104,28 @@ def table_key_gen(key=None, table=ASCII_TABLE):
     The ASCII_TABLE that's provided as a default, is a comprehensive set
     of ascii characters that are legible, unique, and have single octet
     byte representations. This ensures keys can be converted into a
-    cryptographically secure byte stream of binary numbers from 32 to
+    cryptographically secure byte stream of numbers from 32 to
     126.
 
     This generator function provides either deterministic keys from a
-    user key, or generates a random sha_512 hash key and derives a
-    random key with the desired table elements from this hash. The
-    result is a random, normal distribution of characters from among the
-    items within the table. The table argument can also contain an
-    aribtrary set of objects. In this case, this generator function is
-    essentially analogous to one that generates a series of random
-    choices from the table, with the added functionality of allowing
-    deterministic outputs that coincide with the key and size arguments.
-
-    The size parameter determines the number of bytes/elements the
-    output will contain.
+    user key, or generates a random 512-bit hash and derives a random
+    key with the desired table elements from this hash. The result is a
+    random, normal distribution of characters from among the items
+    within the table.
 
     Usage Example:
     key = "hotdiggitydog_thischowisyummy"
-    new_key = ""
-    for char in table_key_gen(key=key)[:60]:
-        new_key += char
+    with table_key_gen(key=key) as generator:
+        new_key = generator()
+        assert new_key != generator()
     print(new_key)
     >>> Hx`4^ej;u&/]qOF21Ea2~(6f"smp'DvMk[(wy'lME%CpCo|1ZWt> &tu=Mw_
     """
-    if key == None:
+    if not key:
         key = csprng()
-    if not is_iterable(table):
-        raise TypeError("table is not iterable")
-    elif isinstance(table, dict):
-        table = list(table.keys())
-    prime = primes[32][0]
-    table_size = len(table)
-    for index in keys(key, key).resize(16).int(16):
-        yield table[index % prime % table_size]
+    size = len(table[:])
+    for key_portion in keys(key, key).int(16).to_base(size, table):
+        yield key_portion
 
 
 async def atable_key(key=None, table=ASCII_TABLE, size=64):
@@ -151,17 +137,17 @@ async def atable_key(key=None, table=ASCII_TABLE, size=64):
     The ASCII_TABLE that's provided as a default, is a comprehensive set
     of ascii characters that are all legible, with unique, single octet
     byte representations. This ensures keys can be converted into byte
-    streams of binary numbers from 32 to 126, with no duplicate values.
+    streams of numbers from 32 to 126, with no duplicate values.
+
+    This generator function provides either deterministic keys from a
+    user key, or generates a random 512-bit hash and derives a random
+    key with the desired table elements from this hash. The result is a
+    random, normal distribution of characters from among the items
+    within the table.
+
+    The size parameter determines the number of bytes/elements the
+    output will contain.
 
-    This function provides either deterministic keys from a user key,
-    or generates a random sha_512 hash key and derives a random key with
-    the desired table elements from this hash. The result is a random,
-    normal distribution of characters from among the items within
-    the table. The table argument can also contain an aribtrary set of
-    objects. In this case, this function is essentially analogous to
-    one that generates list of random choices from the table, with the
-    added functionality of allowing deterministic outputs that coincide
-    with the key and size arguments.
 
     Usage Examples:
 
@@ -174,14 +160,11 @@ async def atable_key(key=None, table=ASCII_TABLE, size=64):
     print(new_key)
     >>> #mE)bOQD@lY%]Qwpb9Zi^32]jteVg
     """
-    async with atable_key_gen(key=key, table=table)[:size] as generator:
-        if type(table) == dict:
-            return await generator.alist()
-        else:
-            try:
-                return await generator.ajoin()
-            except TypeError:
-                return await generator.ajoin(b"")
+    async with atable_key_gen(key=key, table=table) as generator:
+        new_key = await generator()
+        while len(new_key) < size:
+            new_key += await generator()
+        return new_key[:size]
 
 
 def table_key(key=None, table=ASCII_TABLE, size=64):
@@ -193,17 +176,13 @@ def table_key(key=None, table=ASCII_TABLE, size=64):
     The ASCII_TABLE that's provided as a default, is a comprehensive set
     of ascii characters that are all legible, with unique, single octet
     byte representations. This ensures keys can be converted into byte
-    streams of binary numbers from 32 to 126, with no duplicate values.
+    streams of numbers from 32 to 126, with no duplicate values.
 
-    This function provides either deterministic keys from a user key,
-    or generates a random sha_512 hash key and derives a random key with
-    the desired table elements from this hash. The result is a random,
-    normal distribution of characters from among the items within
-    the table. The table argument can also contain an aribtrary set of
-    objects. In this case, this function is essentially analogous to
-    one that generates list of random choices from the table, with the
-    added functionality of allowing deterministic outputs that coincide
-    with the key and size arguments.
+    This generator function provides either deterministic keys from a
+    user key, or generates a random 512-bit hash and derives a random
+    key with the desired table elements from this hash. The result is a
+    random, normal distribution of characters from among the items
+    within the table.
 
     The size parameter determines the number of bytes/elements the
     output will contain.
@@ -219,14 +198,11 @@ def table_key(key=None, table=ASCII_TABLE, size=64):
     print(new_key)
     >>> #mE)bOQD@lY%]Qwpb9Zi^32]jteVg
     """
-    with table_key_gen(key=key, table=table)[:size] as generator:
-        if type(table) == dict:
-            return generator.list()
-        else:
-            try:
-                return generator.join()
-            except TypeError:
-                return generator.join(b"")
+    with table_key_gen(key=key, table=table) as generator:
+        new_key = generator()
+        while len(new_key) < size:
+            new_key += generator()
+        return new_key[:size]
 
 
 @comprehension()
@@ -241,7 +217,7 @@ async def amnemonic(key, salt=None, words=WORD_LIST):
     """
     translate = None
     length = len(words)
-    salt = salt if salt else await asalt()
+    salt = salt if salt else await acsprng()
     key = await apasscrypt(key, salt)
     entropy = abytes_keys(key, salt, key)
     async with entropy.abytes_to_int().arelay(salt) as indexes:
@@ -263,7 +239,7 @@ def mnemonic(key, salt=None, words=WORD_LIST):
     """
     translate = None
     length = len(words)
-    salt = salt if salt else globals()["salt"]()
+    salt = salt if salt else csprng()
     key = passcrypt(key, salt)
     entropy = bytes_keys(key, salt, key)
     with entropy.bytes_to_int().relay(salt) as indexes:
@@ -273,39 +249,42 @@ def mnemonic(key, salt=None, words=WORD_LIST):
             translate = yield words[indexes() % length]
 
 
-async def akeypair(entropy=csprng()):
+async def akeypair(entropy=csprbg()):
     """
-    Returns a pair of symmetric 512-bit hexidecimal keys from our fast
-    cryptographically secure pseudo-random number generator.
+    Returns a pair of symmetric hexidecimal keys from our fast
+    cryptographically secure pseudo-random number generator. One is a
+    512-bit encryption key, the other is a 256-bit salt.
     """
-    return await acsprng(entropy), await acsprng(entropy)
+    return commons.Namespace(
+        key=await acsprng(entropy), salt=await asalt(entropy)
+    )
 
 
-def keypair(entropy=csprng()):
+def keypair(entropy=csprbg()):
     """
-    Returns a pair of symmetric 512-bit hexidecimal keys from our fast
-    cryptographically secure pseudo-random number generator.
+    Returns a pair of symmetric hexidecimal keys from our fast
+    cryptographically secure pseudo-random number generator. One is a
+    512-bit encryption key, the other is a 256-bit salt.
     """
-    return csprng(entropy), csprng(entropy)
+    return commons.Namespace(key=csprng(entropy), salt=salt(entropy))
 
 
 class AsyncKeys:
     """
     This simple class coordinates and manages a symmetric key for
-    establishing an arbitrary number of secure, deterministic
-    streams of key material through an instance's ``__getitem__`` method.
+    establishing an arbitrary number of secure, deterministic streams of
+    key material through an instance's ``__getitem__`` method.
     The class also contains static method key generators which function
     independantly from instance states, as well as the ability to create
     & validate HMAC code.
     """
 
     instance_methods = {
-        akeys, asubkeys, abytes_keys, amnemonic, atable_key, atable_key_gen
+        akeys, abytes_keys, amnemonic, atable_key, atable_key_gen
     }
 
-    aseed = staticmethod(asalt)
+    asalt = staticmethod(asalt)
     akeys = staticmethod(akeys)
-    asubkeys = staticmethod(asubkeys)
     akeypair = staticmethod(akeypair)
     amnemonic = staticmethod(amnemonic)
     apasscrypt = staticmethod(apasscrypt)
@@ -349,6 +328,7 @@ class AsyncKeys:
 
         derived_keystream = keyring.akeys(salt=salt, pid="conversation")
         deciphering = aiootp.aunpack(ciphered).axor(derived_keystream)
+
         async with deciphering.aint_to_ascii() as plaintext:
             assert "Hey, when's the party?" == await plaintext.ajoin()
         """
@@ -412,7 +392,7 @@ class AsyncKeys:
         to implement correctly & is easier to prove guarantees of the
         infeasibility of timing attacks.
         """
-        salt = await atoken_bytes(64)
+        salt = await atoken_bytes(32)
         key = key if key else self.key
         if (
             await asha_256(key, value_0, salt)
@@ -427,7 +407,7 @@ class AsyncKeys:
         Replaces the stored instance key used to create deterministic
         streams of key material &, create & validate HMAC codes.
         """
-        self._key = key if key else await asalt()
+        self._key = key if key else await acsprng()
         for method in self.instance_methods:
             convert_static_method_to_member(
                 self, method.__name__, method, key=self.key,
@@ -453,22 +433,21 @@ class AsyncKeys:
 class Keys:
     """
     This simple class coordinates and manages a symmetric key for
-    establishing an arbitrary number of secure, deterministic
-    streams of key material through an instance's ``__getitem__`` method.
+    establishing an arbitrary number of secure, deterministic streams of
+    key material through an instance's ``__getitem__`` method.
     The class also contains static method key generators which function
     independantly from instance states, as well as the ability to create
     & validate HMAC code.
     """
 
     instance_methods = {
-        keys, subkeys, bytes_keys, mnemonic, table_key, table_key_gen
+        keys, bytes_keys, mnemonic, table_key, table_key_gen
     }
 
-    seed = staticmethod(salt)
+    salt = staticmethod(salt)
     keys = staticmethod(keys)
-    subkeys = staticmethod(subkeys)
     keypair = staticmethod(keypair)
-    amnemonic = staticmethod(amnemonic)
+    mnemonic = staticmethod(mnemonic)
     passcrypt = staticmethod(passcrypt)
     bytes_keys = staticmethod(bytes_keys)
     table_key = staticmethod(table_key)
@@ -506,6 +485,7 @@ class Keys:
 
         derived_keystream = keyring.keys(salt=salt, pid="conversation")
         deciphering = aiootp.unpack(ciphered).xor(derived_keystream)
+
         with deciphering.int_to_ascii() as plaintext:
             assert "Hey, when's the party?" == plaintext.join()
         """
@@ -541,7 +521,7 @@ class Keys:
         This scheme is easier to implement correctly & is easier to
         prove guarantees of the infeasibility of timing attacks. Any
         sync ``hasher`` function can be specified as the HMAC function,
-        which is by default ``sha_256_hmac``
+        which is by default ``sha_256_hmac``.
         """
         if not hmac:
             raise ValueError("``hmac`` argument was not given.")
@@ -565,7 +545,7 @@ class Keys:
         to implement correctly & is easier to prove guarantees of the
         infeasibility of timing attacks.
         """
-        salt = token_bytes(64)
+        salt = token_bytes(32)
         key = key if key else self.key
         if sha_256(key, value_0, salt) == sha_256(key, value_1, salt):
             return True
@@ -577,7 +557,7 @@ class Keys:
         Replaces the stored instance key used to create deterministic
         streams of key material &, create & validate HMAC codes.
         """
-        self._key = key if key else salt()
+        self._key = key if key else csprng()
         for method in self.instance_methods:
             convert_static_method_to_member(
                 self, method.__name__, method, key=self.key,
@@ -605,7 +585,7 @@ async def ainsert_keyrings(self, key=None):
     or instance dictionaries to give those objects access to stateful
     & ephemeral key material generators.
     """
-    key = key if key else await asalt()
+    key = key if key else await acsprng()
     self.keyring = Keys(key=key)
     self.akeyring = AsyncKeys(key=key)
 
@@ -616,7 +596,7 @@ def insert_keyrings(self, key=None):
     or instance dictionaries to give those objects access to stateful
     & ephemeral key material generators.
     """
-    key = key if key else salt()
+    key = key if key else csprng()
     self.keyring = Keys(key=key)
     self.akeyring = AsyncKeys(key=key)
 
@@ -632,8 +612,6 @@ __extras = {
     "akeys": akeys,
     "keys": keys,
     "mnemonic": mnemonic,
-    "asubkeys": asubkeys,
-    "subkeys": subkeys,
     "akeypair": akeypair,
     "keypair": keypair,
     "akeypair_ratchets": akeypair_ratchets,
