@@ -9,9 +9,7 @@
 #
 
 
-import sys
-import pytest
-from pathlib import Path
+from init_tests import *
 
 
 __exports = set()
